// Copyright (C) 2023 WYATT GROUP
// Please see the AUTHORS file for details.
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <https://www.gnu.org/licenses/>.

import 'package:flutter/material.dart';
import 'package:wyatt_ui_components/wyatt_ui_components.dart';
import 'package:wyatt_ui_kit/src/components/cards/widgets/card_background.dart';
import 'package:wyatt_ui_kit/wyatt_ui_kit.dart';

/// {@template card_wrapper}
/// Common wrapper for all cards
/// {@endtemplate}
class CardWrapper extends StatelessWidget {
  /// {@macro card_wrapper}
  CardWrapper({
    required this.child,
    this.radius,
    this.padding,
    this.foregroundColors,
    this.backgroundColors,
    this.borderColors,
    this.stroke,
    this.shadow,
    this.minSize,
    this.maxSize,
    this.background,
    super.key,
  });

  /// Card radius
  ///
  /// Default to `BorderRadius.all(Radius.circular(12.0))`
  final BorderRadiusGeometry? radius;

  /// Padding and gaps of this card
  ///
  /// Default to `Theme.cardTheme.margin`
  final EdgeInsetsGeometry? padding;

  /// Card foreground gradient colors (from left to right)
  ///
  /// Default to `Theme.colorScheme.onPrimary`
  final MultiColor? foregroundColors;

  /// Card background gradient colors (from left to right)
  ///
  /// Default to `Theme.cardTheme.color`
  final MultiColor? backgroundColors;

  /// Border colors (from left to right).
  ///
  /// Default to `null`
  final MultiColor? borderColors;

  /// Stroke of the border
  ///
  /// Default to `null`
  final double? stroke;

  /// Drop shadow
  ///
  /// Default to `null`
  final BoxShadow? shadow;

  /// Minimum size of the card
  ///
  /// Default to `const Size(330, double.infinity)`
  final Size? minSize;

  /// Maximum size of the card
  ///
  /// Default to `const Size(double.infinity, double.infinity)`
  final Size? maxSize;

  /// Background of the card
  final Widget? background;

  final Widget child;

  final GlobalKey _key = GlobalKey();

  Widget _buildChild(BuildContext context, Widget child) {
    final themePadding =
        ThemeHelper.getElement<EdgeInsetsGeometry, EdgeInsetsGeometry>(
      [
        padding,
        Theme.of(context).extension<CardThemeExtension>()?.padding,
        CardThemeExtensionDefault.from(Theme.of(context)).padding,
      ],
    );

    return (background != null)
        ? Stack(
            alignment: Alignment.center,
            children: [
              CardBackground(
                cardKey: _key,
                background: background,
              ),
              Padding(
                padding: themePadding,
                child: child,
              ),
            ],
          )
        : Padding(
            padding: themePadding,
            child: child,
          );
  }

  List<BoxShadow> _shadow(BuildContext context) {
    final themeShadow = ThemeHelper.maybeGetElement<BoxShadow, BoxShadow>(
      [
        shadow,
        Theme.of(context).extension<CardThemeExtension>()?.shadow,
        CardThemeExtensionDefault.from(Theme.of(context)).shadow,
      ],
    );
    return (themeShadow != null) ? [themeShadow] : [];
  }

  @override
  Widget build(BuildContext context) {
    final themeBorderStroke = ThemeHelper.getElement<double, double>(
      [
        stroke,
        Theme.of(context).extension<CardThemeExtension>()?.stroke,
        CardThemeExtensionDefault.from(Theme.of(context)).stroke,
      ],
    );

    final themeBorderRadius =
        ThemeHelper.getElement<BorderRadiusGeometry, BorderRadiusGeometry>(
      [
        radius,
        Theme.of(context).extension<CardThemeExtension>()?.radius,
        CardThemeExtensionDefault.from(Theme.of(context)).radius,
      ],
    );

    final themeMinSize = ThemeHelper.maybeGetElement<Size, Size>(
      [
        minSize,
        Theme.of(context).extension<CardThemeExtension>()?.minSize,
        CardThemeExtensionDefault.from(Theme.of(context)).minSize,
      ],
    );

    final themeMaxSize = ThemeHelper.maybeGetElement<Size, Size>(
      [
        maxSize,
        Theme.of(context).extension<CardThemeExtension>()?.maxSize,
        CardThemeExtensionDefault.from(Theme.of(context)).maxSize,
      ],
    );

    return SelectionArea(
      child: DecoratedBox(
        key: _key,
        decoration: BoxDecoration(
          borderRadius: themeBorderRadius,
          gradient: ThemeHelper.maybeGetElement<MultiColor, Gradient>(
            [
              backgroundColors,
              Theme.of(context)
                  .extension<CardThemeExtension>()
                  ?.backgroundColors,
              CardThemeExtensionDefault.from(Theme.of(context))
                  .backgroundColors,
            ],
            valueValidator: (multiColor) => multiColor?.isGradient,
            transform: (multiColor) =>
                GradientHelper.linearFromMultiColor(multiColor!),
          ),
          color: ThemeHelper.maybeGetElement<MultiColor, Color>(
            [
              backgroundColors,
              Theme.of(context)
                  .extension<CardThemeExtension>()
                  ?.backgroundColors,
              CardThemeExtensionDefault.from(Theme.of(context))
                  .backgroundColors,
            ],
            valueValidator: (multiColor) =>
                multiColor != null && multiColor.isColor,
            transform: (multiColor) => multiColor?.color,
          ),
          border: ThemeHelper.maybeGetElement<MultiColor, BoxBorder>(
            [
              borderColors,
              Theme.of(context).extension<CardThemeExtension>()?.borderColors,
              Theme.of(context)
                  .extension<CardThemeExtension>()
                  ?.backgroundColors,
              CardThemeExtensionDefault.from(Theme.of(context)).borderColors,
            ],
            valueValidator: (multiColor) =>
                multiColor != null && multiColor.isColor,
            transform: (multiColor) {
              if (multiColor!.isGradient) {
                return GradientBoxBorder(
                  gradient: GradientHelper.linearFromMultiColor(multiColor),
                  width: themeBorderStroke,
                );
              }
              return Border.all(
                color: multiColor.color,
                width: themeBorderStroke,
              );
            },
          ),
          boxShadow: _shadow(context),
        ),
        child: (themeMinSize != null || themeMaxSize != null)
            ? ConstrainedBox(
                constraints: BoxConstraints(
                  minWidth: themeMinSize?.width ?? 0,
                  minHeight: themeMinSize?.height ?? 0,
                  maxWidth: themeMaxSize?.width ?? double.infinity,
                  maxHeight: themeMaxSize?.height ?? double.infinity,
                ),
                child: _buildChild(context, child),
              )
            : _buildChild(context, child),
      ),
    );
  }
}
